package ml2java.parser;

import java.io.Reader;
import java.io.FileReader;
import java.io.IOException;
import java.util.NoSuchElementException;
import java.util.Objects;

import ml2java.util.*;

import static ml2java.util.SimpleList.*;
import static ml2java.util.Pair.*;
import static ml2java.util.Option.*;
import static ml2java.util.Bind.*;


public abstract class Operation {
    
    public static Operation Plus() {
        return new Plus();
    }
    public static Operation Minus() {
        return new Minus();
    }
    public static Operation Times() {
        return new Times();
    }
    public static Operation Slash() {
        return new Slash();
    }
    public static Operation Less() {
        return new Less();
    }
    public static Operation Greater() {
        return new Greater();
    }
    public static Operation LessEq() {
        return new LessEq();
    }
    public static Operation GreaterEq() {
        return new GreaterEq();
    }
    public static Operation Equal() {
        return new Equal();
    }
    public static Operation NotEqual() {
        return new NotEqual();
    }
    public static Operation And() {
        return new And();
    }
    public static Operation Or() {
        return new Or();
    }
    
    public abstract <R> R match(Function0<R> f1, Function0<R> f2, Function0<R> f3, Function0<R> f4, Function0<R> f5, Function0<R> f6, Function0<R> f7, Function0<R> f8, Function0<R> f9, Function0<R> f10, Function0<R> f11, Function0<R> f12);
    
    public abstract <R> R accept(OperationVisitor<R> v);
    
    public interface OperationVisitor<R> {
    
        public R visitPlus();
        public R visitMinus();
        public R visitTimes();
        public R visitSlash();
        public R visitLess();
        public R visitGreater();
        public R visitLessEq();
        public R visitGreaterEq();
        public R visitEqual();
        public R visitNotEqual();
        public R visitAnd();
        public R visitOr();
    
    }
    public static abstract class OperationDefaultVisitor<R> implements OperationVisitor<R> {
    
        public R visitPlus() {
            return visitDefault();
        }
        public R visitMinus() {
            return visitDefault();
        }
        public R visitTimes() {
            return visitDefault();
        }
        public R visitSlash() {
            return visitDefault();
        }
        public R visitLess() {
            return visitDefault();
        }
        public R visitGreater() {
            return visitDefault();
        }
        public R visitLessEq() {
            return visitDefault();
        }
        public R visitGreaterEq() {
            return visitDefault();
        }
        public R visitEqual() {
            return visitDefault();
        }
        public R visitNotEqual() {
            return visitDefault();
        }
        public R visitAnd() {
            return visitDefault();
        }
        public R visitOr() {
            return visitDefault();
        }
        public abstract R visitDefault();
    
    }
}

class Plus extends Operation { 
    public Plus() {
    }
    @Override
    public int hashCode() {
        return Objects.hash();
    }
    @Override
    public boolean equals(Object other) {
        if (other == null) return false;
        return (this.getClass() == (other.getClass()));
    }
    public <R> R match(Function0<R> f1, Function0<R> f2, Function0<R> f3, Function0<R> f4, Function0<R> f5, Function0<R> f6, Function0<R> f7, Function0<R> f8, Function0<R> f9, Function0<R> f10, Function0<R> f11, Function0<R> f12) {
        return f1.apply();
    }
    public <R> R accept(OperationVisitor<R> v) {
        return v.visitPlus();
    }
}
class Minus extends Operation { 
    public Minus() {
    }
    @Override
    public int hashCode() {
        return Objects.hash();
    }
    @Override
    public boolean equals(Object other) {
        if (other == null) return false;
        return (this.getClass() == (other.getClass()));
    }
    public <R> R match(Function0<R> f1, Function0<R> f2, Function0<R> f3, Function0<R> f4, Function0<R> f5, Function0<R> f6, Function0<R> f7, Function0<R> f8, Function0<R> f9, Function0<R> f10, Function0<R> f11, Function0<R> f12) {
        return f2.apply();
    }
    public <R> R accept(OperationVisitor<R> v) {
        return v.visitMinus();
    }
}
class Times extends Operation { 
    public Times() {
    }
    @Override
    public int hashCode() {
        return Objects.hash();
    }
    @Override
    public boolean equals(Object other) {
        if (other == null) return false;
        return (this.getClass() == (other.getClass()));
    }
    public <R> R match(Function0<R> f1, Function0<R> f2, Function0<R> f3, Function0<R> f4, Function0<R> f5, Function0<R> f6, Function0<R> f7, Function0<R> f8, Function0<R> f9, Function0<R> f10, Function0<R> f11, Function0<R> f12) {
        return f3.apply();
    }
    public <R> R accept(OperationVisitor<R> v) {
        return v.visitTimes();
    }
}
class Slash extends Operation { 
    public Slash() {
    }
    @Override
    public int hashCode() {
        return Objects.hash();
    }
    @Override
    public boolean equals(Object other) {
        if (other == null) return false;
        return (this.getClass() == (other.getClass()));
    }
    public <R> R match(Function0<R> f1, Function0<R> f2, Function0<R> f3, Function0<R> f4, Function0<R> f5, Function0<R> f6, Function0<R> f7, Function0<R> f8, Function0<R> f9, Function0<R> f10, Function0<R> f11, Function0<R> f12) {
        return f4.apply();
    }
    public <R> R accept(OperationVisitor<R> v) {
        return v.visitSlash();
    }
}
class Less extends Operation { 
    public Less() {
    }
    @Override
    public int hashCode() {
        return Objects.hash();
    }
    @Override
    public boolean equals(Object other) {
        if (other == null) return false;
        return (this.getClass() == (other.getClass()));
    }
    public <R> R match(Function0<R> f1, Function0<R> f2, Function0<R> f3, Function0<R> f4, Function0<R> f5, Function0<R> f6, Function0<R> f7, Function0<R> f8, Function0<R> f9, Function0<R> f10, Function0<R> f11, Function0<R> f12) {
        return f5.apply();
    }
    public <R> R accept(OperationVisitor<R> v) {
        return v.visitLess();
    }
}
class Greater extends Operation { 
    public Greater() {
    }
    @Override
    public int hashCode() {
        return Objects.hash();
    }
    @Override
    public boolean equals(Object other) {
        if (other == null) return false;
        return (this.getClass() == (other.getClass()));
    }
    public <R> R match(Function0<R> f1, Function0<R> f2, Function0<R> f3, Function0<R> f4, Function0<R> f5, Function0<R> f6, Function0<R> f7, Function0<R> f8, Function0<R> f9, Function0<R> f10, Function0<R> f11, Function0<R> f12) {
        return f6.apply();
    }
    public <R> R accept(OperationVisitor<R> v) {
        return v.visitGreater();
    }
}
class LessEq extends Operation { 
    public LessEq() {
    }
    @Override
    public int hashCode() {
        return Objects.hash();
    }
    @Override
    public boolean equals(Object other) {
        if (other == null) return false;
        return (this.getClass() == (other.getClass()));
    }
    public <R> R match(Function0<R> f1, Function0<R> f2, Function0<R> f3, Function0<R> f4, Function0<R> f5, Function0<R> f6, Function0<R> f7, Function0<R> f8, Function0<R> f9, Function0<R> f10, Function0<R> f11, Function0<R> f12) {
        return f7.apply();
    }
    public <R> R accept(OperationVisitor<R> v) {
        return v.visitLessEq();
    }
}
class GreaterEq extends Operation { 
    public GreaterEq() {
    }
    @Override
    public int hashCode() {
        return Objects.hash();
    }
    @Override
    public boolean equals(Object other) {
        if (other == null) return false;
        return (this.getClass() == (other.getClass()));
    }
    public <R> R match(Function0<R> f1, Function0<R> f2, Function0<R> f3, Function0<R> f4, Function0<R> f5, Function0<R> f6, Function0<R> f7, Function0<R> f8, Function0<R> f9, Function0<R> f10, Function0<R> f11, Function0<R> f12) {
        return f8.apply();
    }
    public <R> R accept(OperationVisitor<R> v) {
        return v.visitGreaterEq();
    }
}
class Equal extends Operation { 
    public Equal() {
    }
    @Override
    public int hashCode() {
        return Objects.hash();
    }
    @Override
    public boolean equals(Object other) {
        if (other == null) return false;
        return (this.getClass() == (other.getClass()));
    }
    public <R> R match(Function0<R> f1, Function0<R> f2, Function0<R> f3, Function0<R> f4, Function0<R> f5, Function0<R> f6, Function0<R> f7, Function0<R> f8, Function0<R> f9, Function0<R> f10, Function0<R> f11, Function0<R> f12) {
        return f9.apply();
    }
    public <R> R accept(OperationVisitor<R> v) {
        return v.visitEqual();
    }
}
class NotEqual extends Operation { 
    public NotEqual() {
    }
    @Override
    public int hashCode() {
        return Objects.hash();
    }
    @Override
    public boolean equals(Object other) {
        if (other == null) return false;
        return (this.getClass() == (other.getClass()));
    }
    public <R> R match(Function0<R> f1, Function0<R> f2, Function0<R> f3, Function0<R> f4, Function0<R> f5, Function0<R> f6, Function0<R> f7, Function0<R> f8, Function0<R> f9, Function0<R> f10, Function0<R> f11, Function0<R> f12) {
        return f10.apply();
    }
    public <R> R accept(OperationVisitor<R> v) {
        return v.visitNotEqual();
    }
}
class And extends Operation { 
    public And() {
    }
    @Override
    public int hashCode() {
        return Objects.hash();
    }
    @Override
    public boolean equals(Object other) {
        if (other == null) return false;
        return (this.getClass() == (other.getClass()));
    }
    public <R> R match(Function0<R> f1, Function0<R> f2, Function0<R> f3, Function0<R> f4, Function0<R> f5, Function0<R> f6, Function0<R> f7, Function0<R> f8, Function0<R> f9, Function0<R> f10, Function0<R> f11, Function0<R> f12) {
        return f11.apply();
    }
    public <R> R accept(OperationVisitor<R> v) {
        return v.visitAnd();
    }
}
class Or extends Operation { 
    public Or() {
    }
    @Override
    public int hashCode() {
        return Objects.hash();
    }
    @Override
    public boolean equals(Object other) {
        if (other == null) return false;
        return (this.getClass() == (other.getClass()));
    }
    public <R> R match(Function0<R> f1, Function0<R> f2, Function0<R> f3, Function0<R> f4, Function0<R> f5, Function0<R> f6, Function0<R> f7, Function0<R> f8, Function0<R> f9, Function0<R> f10, Function0<R> f11, Function0<R> f12) {
        return f12.apply();
    }
    public <R> R accept(OperationVisitor<R> v) {
        return v.visitOr();
    }
}

