package ml2java.parser;

import java.io.Reader;
import java.io.FileReader;
import java.io.IOException;
import java.util.NoSuchElementException;
import java.util.Objects;

import ml2java.util.*;

import static ml2java.util.SimpleList.*;
import static ml2java.util.Pair.*;
import static ml2java.util.Option.*;
import static ml2java.util.Bind.*;


public class Util {

    
    public static <A> A nth(SimpleList<A> l, Integer n) {
        return match(l,
            /* Nil */ () -> {
                throw new NoSuchElementException();
            },
            /* Cons */ (h, t) -> {
                if (n.equals(0)) {
                    return h;
                } else {
                    return nth(t, (n - 1));
                }
            });
    }
    
    public static <A, B> SimpleList<Pair<A, B>> combine(SimpleList<A> l1, SimpleList<B> l2) {
        return match(l1,
            /* Nil */ () -> {
                return match(l2,
                    /* Nil */ () -> {
                        return Nil();
                    },
                    /* Cons */ (_3, _2) -> {
                        return Nil();
                    });
            },
            /* Cons */ (h1, t1) -> {
                return match(l2,
                    /* Nil */ () -> {
                        return Nil();
                    },
                    /* Cons */ (h2, t2) -> {
                        return Cons(Pair(h1, h2), combine(t1, t2));
                    });
            });
    }
    
    public static <A, B> Pair<SimpleList<A>, SimpleList<B>> split(SimpleList<Pair<A, B>> l) {
        return match(l,
            /* Nil */ () -> {
                Pair<SimpleList<A>, SimpleList<B>> r = Pair(Nil(), Nil());
                return r;
            },
            /* Cons */ (h, t) -> {
                return split(t).match((l1, l2) -> {
                    return h.match((h1, h2) -> {
                        return Pair(Cons(h1, l1), Cons(h2, l2));
                    });
                });
            });
    }
    
    public static <B, A> B assoc(SimpleList<Pair<A, B>> l, A x) {
        return match(l,
            /* Nil */ () -> {
                throw new NoSuchElementException();
            },
            /* Cons */ (h, t) -> {
                return h.match((k, v) -> {
                    if (x.equals(k)) {
                        return v;
                    } else {
                        return assoc(t, x);
                    }
                });
            });
    }
    
    public static <B, A> Boolean mem_assoc(SimpleList<Pair<A, B>> l, A x) {
        return match(l,
            /* Nil */ () -> {
                return false;
            },
            /* Cons */ (h, t) -> {
                return h.match((k, _1) -> {
                    if (x.equals(k)) {
                        return true;
                    } else {
                        return mem_assoc(t, x);
                    }
                });
            });
    }
    
    public static <B, A> Option<B> find(SimpleList<Pair<A, B>> l, A x) {
        return match(l,
            /* Nil */ () -> {
                return None();
            },
            /* Cons */ (h, t) -> {
                return h.match((k, v) -> {
                    if (x.equals(k)) {
                        return Some(v);
                    } else {
                        return find(t, x);
                    }
                });
            });
    }
    
    public static <B, A> SimpleList<B> map(Function1<A, B> f, SimpleList<A> l) {
        return match(l,
            /* Nil */ () -> {
                return Nil();
            },
            /* Cons */ (h, t) -> {
                return Cons(f.apply(h), map(f, t));
            });
    }
    
    public static <A> SimpleList<A> reverse(SimpleList<A> l) {
        return match(l,
            /* Nil */ () -> {
                return Nil();
            },
            /* Cons */ (h, t) -> {
                return concat(reverse(t), single(h));
            });
    }
    public static Function2<Function1<Integer, Boolean>, SimpleList<Integer>, SimpleList<Boolean>> map3 = (Function1<Integer, Boolean> f, SimpleList<Integer> l) -> {
            return map(f, l);
        };
    
}
